#!/usr/bin/python
#
# Copyright 2015 Microsoft Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import argparse
import array
import hashlib
import inspect
import datetime
import os
import os.path
import stat
import string
import sys
import syslog
from defusedxml.ElementTree import parse
from daemon import runner

import azsec.scanmanager as scanmanager
import azsec.util.common as util
import azsec.util.config as cfgutil

# Parse command line arguments, set options as appropriate
def ParseArguments():
    parser = argparse.ArgumentParser(description='Scan the system for security threats and vulnerability')
    parser.add_argument(
        '-v', '--verbose', help='write output to stdout', action='count')
    parser.add_argument(
        '-m', '--manual', help='perform scan manually', action='store_true')
    parser.add_argument(
        '-n', '--name', help='specify scanner by module name ("azsec.baselinescan.bscan" or "bscan")')
    parser.add_argument(
        '-start', '--startDaemon', help='start scanningdaemon', action='store_true')
    parser.add_argument(
        '-stop', '--stopDaemon', help='stop scanningdaemon', action='store_true')
    parser.add_argument(
        '-restart', '--restartDaemon', help='restart scanningdaemon', action='store_true')
    args = parser.parse_args()
    # Set verbosity level for writing to console
    if args.verbose:
        util.runtimeVerbosity = args.verbose
    return args

# Try initiating a scan based on options provided
# Return true/false if scan was/was not initiated
# Return num findings from the scans
def PerformManualScan(args):
    syslog.openlog(facility=cfgutil.defaultLogFacility)
    scanner = scanmanager.scanmanager()
    while True:
        if not scanner.InitScan():
            exitCode = 1
            break
        if args.name:
            module = cfgutil.FindScannerName(args.name, scanner.scanConfigMap)
            if module is None:
                exitCode = 1
                break
            scansToRun = [module]
        else:
            lastScan = {}
            for scanName in scanner.scanConfigMap.keys():
                lastScan[scanName] = scanmanager.epoch
            scansToRun = scanner.DetermineWhichScansToRun(lastScan, datetime.datetime.now())
        exitCode = scanner.PerformScan(scansToRun)
        break
    syslog.closelog()
    return exitCode

# Try to start/stop/restart the daemon
def TryManageDaemon(args):
    # If invalid param is specified, return
    if (len(sys.argv) < 2):
        return
    # Determine which daemon operation is being attempted and set sys.argv[1] appropriately
    if args.startDaemon:
        sys.argv[1] = "start"
    elif args.stopDaemon:
        sys.argv[1] = "stop"
    elif args.restartDaemon:
        sys.argv[1] = "restart"
    else:
        # No valid daemon options specified
        return
    syslog.openlog(facility=cfgutil.defaultLogFacility)
    util.WriteLog(syslog.LOG_INFO, "Attempting to {0} the daemon".format(sys.argv[1]))
    syslog.closelog()
    try:
        scanner = scanmanager.scanmanager()
        daemonRunner = runner.DaemonRunner(scanner)
        daemonRunner.do_action()
        sys.exit(0)
    except Exception:
        print "An exception occurred while attempting to '{0}' the daemon.".format(sys.argv[1])
        print "Confirm you have appropriate permissions and that the daemon is in the expected state"
        sys.exit(1)

# Parse arguments and config file, kick off scans, report results
def main():
    args = ParseArguments()
    # If parameters indicate we should run a manual scan, do run it
    if args and args.manual:
        exitCode = PerformManualScan(args)
        sys.exit(exitCode)
    TryManageDaemon(args)
    # If we got this far, the daemon isn't running
    print "No valid options specified. Run " + __file__ + " -h for help"
    sys.exit(1)

main()
