#!/usr/bin/python
#
# Copyright 2015 Microsoft Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import argparse
import array
import datetime
import dateutil.parser
import hashlib
import inspect
import isodate
import os
import os.path
import stat
import string
import sys
import syslog
import time
import xml.etree.ElementTree as elementTree

import azsec.util.common as util
import azsec.util.config as cfgutil
from defusedxml.ElementTree import parse

# Initialize waagent
util.waagent.LoggerInit('/var/log/waagent.log','/dev/null')

# General Settings
lastScanFile = util.pkgConfigDir + "/lastScan.xml"
epoch = dateutil.parser.parse('1970-01-01 00:00:00')

class scanmanager():
    scanConfigMap = {}

    def __init__(self):
        pidPath='/var/run/azsecd'
        self.stdin_path = '/dev/null'
        self.stdout_path = '/dev/null'
        self.stderr_path = '/dev/null'
        self.pidfile_timeout = 3
        if not os.path.exists(pidPath):
            os.makedirs(pidPath)
        self.pidfile_path = "{0}/azsecd.pid".format(pidPath)

    def InitScan(self):
        success, self.scanConfigMap = cfgutil.ParseConfig(util.scanConfigDir, loadModule=True)
        if not success:
            util.WriteConsole(syslog.LOG_ERR, "Scanner configuration error(s) in "+util.scanConfigDir)
        if not self.scanConfigMap:
            util.WriteConsole(syslog.LOG_ERR, "Could not parse a usable scan schedule from "+util.scanConfigDir)
            return False
        return True

    def TryScan(self):
        util.WriteLog(syslog.LOG_INFO, "Attempting to perform scheduled scans")
        if not self.InitScan():
            return False
        success, lastScan = self.ParseLastScan(lastScanFile)
        if not success:
            util.WriteConsole(syslog.LOG_WARNING, "Could not parse last scan times from {0}, will assume this is the first run".format(lastScanFile))
            self.CreateDefaultLastScanFile()
            return False
        # Determine which scans to run
        now = datetime.datetime.now()
        scansToRun = self.DetermineWhichScansToRun(lastScan, now)
        # Run Scans
        retval = False
        if len(scansToRun) > 0:
            retval = self.PerformScan(scansToRun) == 0
            # Reset latest scan times for any scans that were run, write to file
            for scanName in scansToRun:
                lastScan[scanName] = now
            self.CreateLastScanFile(lastScan, lastScanFile)
        return retval

    def DetermineWhichScansToRun(self, lastScan, now):
        return [m for m in self.scanConfigMap.keys() if not self.scanConfigMap[m].IsDisabled() and now - lastScan[m] >= self.scanConfigMap[m].duration]

    def CreateDefaultLastScanFile(self, outFile = lastScanFile):
        lastScan = {}
        for scanName in self.scanConfigMap.keys():
            lastScan[scanName] = epoch
        return self.CreateLastScanFile(lastScan, outFile)

    def CreateLastScanFile(self, lastScanTimes, outFile = lastScanFile):
        root = elementTree.Element("lastScan")
        scanners = elementTree.SubElement(root, "scanners")
        for scanName in self.scanConfigMap.keys():
            scanners.set(scanName, cfgutil.FormatDatetime(lastScanTimes[scanName]))
        return util.CreateXmlFile(outFile, elementTree.ElementTree(root), "644")

    def ParseLastScan(self, inFile=lastScanFile):
        success, elementTree = util.GetXmlFileElementTree(inFile)
        if not success:
            return False, None
        root = elementTree.getroot()
        scanners = root.find('scanners')
        # Iterate through values, if a value doesn't exist, create it
        lastScanTimes = {}
        for scanName in self.scanConfigMap.keys():
            lastScanTimes[scanName] = self.ParseConfigEntryToDateTime(scanners, scanName)
        return True, lastScanTimes

    def ParseConfigEntryToDateTime(self, scanners, scanName):
        if scanners is None or scanName is None:
            # If no valid object specified, parse as Epoch time
            return epoch
        try:
            return dateutil.parser.parse(scanners.get(scanName))
        except:
            util.WriteLog(syslog.LOG_ERR, "Unable to parse a datetime entry from last scan file")
        return epoch

    def PerformScan(self, selectedScans):
        if selectedScans is None:
            util.WriteLog(syslog.LOG_ERR, "Invalid object was passed when initiating a scan")
            return 1
        findings = 0
        attemptedScans = 0
        errCode = 0
        for scanName in selectedScans:
            try:
                scanConfig = self.scanConfigMap[scanName]
                e,f = scanConfig.funcRunScan(scanConfig.logFacility, scanConfig.args)
                errCode += e
                findings += f
                if scanConfig.remediate:
                    errCode += scanConfig.funcRemediate(scanConfig.logFacility, scanConfig.args)
                attemptedScans += 1
            except:
                util.WriteLog(syslog.LOG_ERR, "[{0}] Run-time error".format(scanConfig))
                errCode += 1
        logpri = syslog.LOG_WARNING if errCode else syslog.LOG_INFO
        util.WriteConsole(logpri, "Completed " + `attemptedScans` + " scan(s) with " + `errCode` + " errors and " + `findings` + " findings")
        return errCode

    def run(self):
        # If running as a daemon, prevent print statements from spewing to stdout
        sys.stdout = open('/dev/null', 'w')
        while True:
            self.TryScan()
            time.sleep(300)
