#!/usr/bin/python
# Contains utility methods that are available with the default python installation
# These are available to the handler with no modification to the system
#
# Copyright 2015 Microsoft Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import imp
import isodate
import json
import os
import os.path
import platform
import re
import syslog
from defusedxml.ElementTree import parse
from jsonschema import validate

pkgConfigDir="/etc/azsec"
scanConfigDir = pkgConfigDir + "/cfg"
ubuntu = "Ubuntu"
runtimeVerbosity = 0
# Scans can be no more frequent than every 5 minutes
minimumDurationSeconds = 300

def StringIsNoneOrEmpty(str):
    if str is None or len(str) < 1:
        return True
    return False

def searchWAAgent():
    """ Load the waagent and expose it as a submodule """
    agentPath = '/usr/sbin/waagent'
    if(os.path.isfile(agentPath)):
        return agentPath
    user_paths = os.environ['PYTHONPATH'].split(os.pathsep)
    for user_path in user_paths:
        agentPath = os.path.join(user_path, 'waagent')
        if(os.path.isfile(agentPath)):
            return agentPath
    return None

def DistInfo(fullname=0):
    """ Define the function in case waagent(<2.0.4) doesn't have DistInfo() """
    if 'FreeBSD' in platform.system():
        release = re.sub('\-.*\Z', '', str(platform.release()))
        distinfo = ['FreeBSD', release]
        return distinfo
    if os.path.isfile('/etc/oracle-release'):
        release = re.sub('\-.*\Z', '', str(platform.release()))
        distinfo = ['Oracle', release]
        return distinfo
    if 'linux_distribution' in dir(platform):
        distinfo = list(platform.linux_distribution(\
                        full_distribution_name=fullname))
        # remove trailing whitespace in distro name
        distinfo[0] = distinfo[0].strip()
        return distinfo
    else:
        return platform.dist()

def ReadJsonFile(filePath):
    """ Attempt to parse the json file,
    return true if successful, false if not"""
    if not filePath:
        error = "Empty string cannot be parsed as a filepath"
        return False, None
    if not os.path.exists(filePath):
        error = "File {0} does not exist".format(filePath)
        return False, None
    try:
        with open(filePath, 'r') as file:
            result = json.load(file)
            return True, result
    except:
        error = "Failed to parse JSON from {0}".format(filePath)
        return False, None

def GetXmlFileElementTree(configFile):
    """ Open the specified file, parse as xml, and return the element tree object """
    if not configFile:
        WriteConsole(syslog.LOG_ERR, "Config file path must be a valid string")
        return False, 0
    if not os.path.isfile(configFile):
        WriteConsole(syslog.LOG_ERR, "No config file exists at the path " + configFile)
        return False, 0
    # Parse the file
    try:
        WriteVerbose("Parsing config file " + configFile)
        elementTree = parse(configFile)
        return True, elementTree
    except:
        WriteConsole(syslog.LOG_ERR, "Unable to parse file as XML: " +  configFile)
        return False, 0

def CreateXmlFile(path, tree, mode):
    try:
        fd = open(path, 'w')
        tree.write(fd)
        fd.close()
        os.system('chmod {0} {1}'.format(mode, path))
        return os.path.exists(path)
    except:
        WriteConsole(syslog.LOG_ERR, "Failed to create " + path)
        return False

# 0 = Default message verbosity (-v), write to syslog
# 1 = Verbose output (-vv), never write to syslog
# 2 = Debug output (-vvv), never write to syslog
def WriteLog(level, message, messageVerbosity=0):
    if (len(message) < 0):
        return
    if (messageVerbosity < 1):
        syslog.syslog(level, message)
    if (messageVerbosity < runtimeVerbosity):
        print message

# Writes the specified message with a verbosity that is guaranteed to write to console
def WriteConsole(level, message):
    WriteLog(level, message, -1)

# Writes the specified message with verbosity level 1 (-vv)
def WriteVerbose(message):
    WriteLog(syslog.LOG_INFO, message, 1)

# Writes the specified message with verbosity level 2 (-vvv)
def WriteDebug(message):
    WriteLog(syslog.LOG_INFO, message, 2)

distro = DistInfo()[0]
agentPath = searchWAAgent()
if(agentPath):
    waagent = imp.load_source('waagent', agentPath)
else:
    raise Exception("Can't load waagent.")
