#!/usr/bin/python
#
# Copyright 2015 Microsoft Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import os
import os.path
import sys
import syslog
import time
import isodate
import inspect
import importlib
import azsec.util.common as util
from defusedxml.ElementTree import parse
from os.path import isfile, join
import xml.etree.ElementTree as etree

defaultLogFacility = syslog.LOG_AUTH

SCANNER = "scanner"
MODULE = "module"
CYCLE = "cycle"
LOG_FACILITY = "log"
REMEDIATE = "remediate"
ARG = "arg"
DISABLED = "disabled"
FUNC_RUN_SCAN = "RunScan"
FUNC_REMEDIATE = "Remediate"
FUNC_SIGNATURE = ['logFacility', 'args']

def IsStateEqual(proposedState, constState):
    """Return true if the string 'constState' equal to 'proposedState' in a case insensitive match"""
    if proposedState is None or constState is None:
        return False
    return constState.lower() == proposedState.lower()

def FormatDatetime(dateTime):
    """Turn the provided datetime object into a formatted string"""
    if dateTime is None:
        return ''
    return dateTime.strftime('%Y-%m-%d %H:%M:%S')

def SetScanMethods(scanconfig):
    try:
        mod = importlib.import_module(scanconfig.module)
        scanconfig.funcRunScan = getattr(mod, FUNC_RUN_SCAN)
        if inspect.getargspec(scanconfig.funcRunScan)[0] != FUNC_SIGNATURE:
            util.WriteLog(syslog.LOG_ERR, "[{0}] {1} signature does not match {2}".format(scanconfig.module, FUNC_RUN_SCAN, FUNC_SIGNATURE))
            return False
        if scanconfig.remediate:
            scanconfig.funcRemediate = getattr(mod, FUNC_REMEDIATE)
            if inspect.getargspec(scanconfig.funcRemediate)[0] != FUNC_SIGNATURE:
                util.WriteLog(syslog.LOG_ERR, "[{0}] {1} signature does not match {2}".format(scanconfig.module, FUNC_REMEDIATE, FUNC_SIGNATURE))
                return False
        return True
    except ImportError as ex:
        util.WriteLog(syslog.LOG_ERR, "ImportError for {0}: {1}".format(scanconfig.module, ex))
    except AttributeError as ex:
        util.WriteLog(syslog.LOG_ERR, "AttributeError for {0}: {1}".format(scanconfig.module, ex))
    except:
        util.WriteLog(syslog.LOG_ERR, "{0}: {1}".format(scanconfig.module, sys.exc_info()[0]))
    return False

def ParseISO8601ToDuration(isoDuration):
    """Parse the provided string into a time duration"""
    # Fail if not a parseable string
    if not isoDuration:
        return None
    try:
        duration = isodate.parse_duration(isoDuration)
        return duration
    except:
        return None

def ParseSyslogFacility(module, log):
    if log is None:
        return True, defaultLogFacility
    try:
        # auth -> LOG_AUTH
        return True, getattr(syslog, "LOG_{0}".format(log.upper()))
    except:
        util.WriteLog(syslog.LOG_ERR, "{0}: unsupported log facility {1}, assuming default".format(module, log))
        return False, defaultLogFacility

def ParseScanner(elementTree, configFile):
    try:
        scanner = elementTree.getroot()
        if scanner.tag != SCANNER:
            util.WriteLog(syslog.LOG_ERR, "unsupported scanner tag in {0}".format(configFile))
            return None
        module = scanner.get(MODULE)
        if module is None:
            util.WriteLog(syslog.LOG_ERR, "scanner module name is missing in {0}".format(configFile))
            return None
        cycle = scanner.get(CYCLE)
        if cycle is None:
            util.WriteLog(syslog.LOG_ERR, "scanner cycle is missing for {0}, assuming disabled".format(module))
        args = {}
        for arg in scanner.iter(ARG):
            args.update(dict(arg.attrib))
        return scanconfig(module, cycle, scanner.get(LOG_FACILITY), scanner.get(REMEDIATE), args, configFile)
    except:
        util.WriteLog(syslog.LOG_ERR, "error in parsing scanner config {0}".format(configFile))
        return None

def ParseConfig(scanConfigDir, loadModule = False):
    """Parse scanner config files provided directory name.
    For each scanner validate its configuration parameters, load the module and verify scan API"""
    if not scanConfigDir:
        util.WriteLog(syslog.LOG_ERR, "Config directory must be a valid string")
        return False, {}
    if not os.path.isdir(scanConfigDir):
        util.WriteLog(syslog.LOG_ERR, "No config directory '{0}'".format(scanConfigDir))
        return False, {}
    retval = True
    scanConfigMap = {}
    for configFile in [join(scanConfigDir, f) for f in os.listdir(scanConfigDir) if isfile(join(scanConfigDir, f))]:
        success, elementTree = util.GetXmlFileElementTree(configFile)
        if not success:
            retval = False
            continue
        scanconfig = ParseScanner(elementTree, configFile)
        if scanconfig is None:
            retval = False
            continue
        # load the module and get the scan methods
        if loadModule and not SetScanMethods(scanconfig):
            retval = False
            continue
        scanConfigMap[scanconfig.module] = scanconfig
    # disable scanners that are no longer in the configuration
    return retval, scanConfigMap

def ModifyConfig(scanConfigUpdate, scanConfig):
    """ Update config parameters with new values. If a new value is None, keep the default.
    Save the changes to the scanner configuration file """
    if not (isinstance(scanConfigUpdate, scanconfig) and isinstance(scanConfig, scanconfig) and scanConfigUpdate.module == scanConfig.module):
        util.WriteConsole(syslog.LOG_ERR, "An invalid parameter was passed while attempting to modify config")
        return False
    util.WriteDebug("Processing request for " + scanConfig.module)
    if scanConfig.fname is None:
        util.WriteConsole(syslog.LOG_ERR, "[{0}] Scanner configuration file is not set".format(scanConfig.module))
        return False
    if scanConfig.Update(scanConfigUpdate):
        return scanConfig.WriteToFile()
    return True

def ModifyConfigs(configList, scanConfigDir):
    """ Attempts to modify the specified configs. If a change occurs, flush to file """
    scanConfigMap = {}
    if not os.path.isdir(scanConfigDir):
        return 1
    success, scanConfigMap = ParseConfig(scanConfigDir)
    if not success:
        return 1
    ret = 0
    for scanConfig in configList:
        if scanConfig.module in scanConfigMap:
            if not ModifyConfig(scanConfig, scanConfigMap[scanConfig.module]):
                util.WriteConsole(syslog.LOG_ERR, "[{0}] Failed to modify scanner config".format(scanConfig.module))
                ret += 1
    return ret

def FindScannerName(subtxt, scanConfigMap):
    if scanConfigMap is None:
        return None
    module = None
    for key in scanConfigMap.keys():
        if key.find(subtxt) != -1:
            if module is None:
                module = key
            else:
                util.WriteConsole(syslog.LOG_ERR, "'{0}' ambiguous matching in {1}, {2}".format(subtxt, module, key))
                return None
    if module is None:
        util.WriteConsole(syslog.LOG_ERR, "'{0}' does not match any existing module".format(subtxt))
    return module

class scanconfig():
    # static scanner config parameters from the configuration file
    module = None
    cycle = None
    log = None
    remediate = False
    args = {}
    # run-time variables
    fname = None
    duration = None
    logFacility = defaultLogFacility
    funcRunScan = None
    funcRemediate = None

    def __init__(self, module, cycle, log, remediate, args, fname = None):
        self.module = module
        self.SetDuration(cycle)
        self.SetLogFacility(log)
        self.SetRemediate(remediate)
        if args is not None:
            self.args = args
        self.fname = fname

    def IsValid(self):
        return self.module is not None

    def Disable(self):
        self.cycle = DISABLED
        self.duration = None

    def IsDisabled(self):
        return self.duration is None

    def SetDuration(self, cycle):
        """Update scanner cycle. Return True if the value has changed, False otherwise"""
        if self.cycle == cycle:
            return False
        self.cycle = cycle
        if IsStateEqual(cycle, DISABLED):
            self.duration = None
        else:
            self.duration = ParseISO8601ToDuration(cycle)
            if self.duration is None:
                util.WriteLog(syslog.LOG_ERR, "[{0}] could not parse cycle '{1}', assuming disabled".format(self.module, cycle))
                self.cycle = DISABLED
        return True

    def SetLogFacility(self, log):
        """Update scanner log facility. Return True if the value has changed, False otherwise"""
        if self.log == log:
            return False
        self.log = log
        success, self.logFacility = ParseSyslogFacility(self.module, log)
        if not success:
            self.log = None
        return True

    def SetRemediate(self, remediate):
        """Update scanner remediate flag. Return True if the value has changed, False otherwise"""
        bVal = isinstance(remediate, basestring) and remediate.lower() == "true"
        if bVal == self.remediate:
            return False
        self.remediate = bVal
        return True

    def Update(self, update):
        """Update scanner configuration parameters. Return True if any parameter has changed"""
        # make sure the module matches
        if not self.IsValid() or self.module != update.module:
            return False
        # compare and update
        modified = self.SetDuration(update.cycle)
        modified |= self.SetLogFacility(update.log)
        modified |= self.SetRemediate(str(update.remediate))
        if cmp(self.args, update.args) != 0:
            modified = True
            self.args = update.args
        return modified

    def WriteToFile(self):
        if not self.IsValid() or self.fname is None:
            return False
        root = etree.Element(SCANNER)
        root.set(MODULE, self.module)
        root.set(CYCLE, self.cycle if self.cycle is not None else DISABLED)
        if self.log is not None:
            root.set(LOG_FACILITY, self.log)
        root.set(REMEDIATE, str(self.remediate))
        scanArgs = etree.SubElement(root, ARG)
        for argName, argValue in self.args.iteritems():
            scanArgs.set(argName, argValue)
        return util.CreateXmlFile(self.fname, etree.ElementTree(root), "644")

    def toString(self):
        return "Scanner config module:{0}\n\tcycle:{1}({2}) log:{3}({4}) remediate:{5}\n\targs:{6}\n\tfilename:{7}".format(self.module, self.cycle, self.duration, self.log, self.logFacility, self.remediate, self.args, self.fname)